@@ -1,4 +1,4 @@
-#!/usr/local/bin/php-7.4
+#!/usr/local/bin/php-8.0
 <?php
 
 /*
@@ -41,7 +41,7 @@ function _die($msg, $usage=false)
 $rcmail = rcube::get_instance();
 
 // get arguments
-$args = rcube_utils::get_opt(array('h' => 'host', 'a' => 'age', 'd' => 'dry-run:bool'));
+$args = rcube_utils::get_opt(['h' => 'host', 'a' => 'age', 'd' => 'dry-run:bool']);
 
 if (!empty($args['age']) && ($age = intval($args['age']))) {
     $db = $rcmail->get_dbh();
@@ -57,12 +57,12 @@ if (!empty($args['age']) && ($age = intval($args['age']))) {
             printf("%s (%s)\n", $user['username'], $user['mail_host']);
             continue;
         }
-        system(sprintf("php %s/deluser.sh --host=%s %s", INSTALL_PATH . 'bin', $user['mail_host'], $user['username']));
+        system(sprintf("php %s/deluser.sh --host=%s %s", INSTALL_PATH . 'bin', escapeshellarg($user['mail_host']), escapeshellarg($user['username'])));
     }
-    exit(1);
+    exit(0);
 }
 
-$username = trim($args[0]);
+$username = isset($args[0]) ? trim($args[0]) : null;
 if (empty($username)) {
     _die("Missing required parameters", true);
 }
@@ -103,7 +103,7 @@ if (!$user) {
 }
 
 // inform plugins about approaching user deletion
-$plugin = $rcmail->plugins->exec_hook('user_delete_prepare', array('user' => $user, 'username' => $username, 'host' => $args['host']));
+$plugin = $rcmail->plugins->exec_hook('user_delete_prepare', ['user' => $user, 'username' => $username, 'host' => $args['host']]);
 
 // let plugins cleanup their own user-related data
 if (!$plugin['abort']) {
@@ -120,7 +120,7 @@ if ($plugin['abort']) {
 
 // deleting the user record should be sufficient due to ON DELETE CASCADE foreign key references
 // but not all database backends actually support this so let's do it by hand
-foreach (array('identities','contacts','contactgroups','dictionary','cache','cache_index','cache_messages','cache_thread','searches','users') as $table) {
+foreach (['identities','contacts','contactgroups','dictionary','cache','cache_index','cache_messages','cache_thread','searches','users'] as $table) {
     $db->query('DELETE FROM ' . $db->table_name($table, true) . ' WHERE `user_id` = ?', $user->ID);
 }
 
