@@ -1,4 +1,4 @@
-#!/usr/local/bin/php-7.4
+#!/usr/local/bin/php-8.0
 <?php
 /*
  +-----------------------------------------------------------------------+
@@ -23,9 +23,9 @@ define('INSTALL_PATH', realpath(__DIR__ . '/..') . '/' );
 require_once INSTALL_PATH . 'program/include/clisetup.php';
 
 // get arguments
-$opts = rcube_utils::get_opt(array(
+$opts = rcube_utils::get_opt([
     'd' => 'dir',
-));
+]);
 
 if (empty($opts['dir'])) {
     rcube::raise_error("Database schema directory not specified (--dir).", false, true);
