@@ -1,99 +1,119 @@
 <?php
 
-if (!class_exists('rcmail_install', false) || !is_object($RCI)) {
+/**
+ +-----------------------------------------------------------------------+
+ | This file is part of the Roundcube Webmail client                     |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
+ |                                                                       |
+ | Licensed under the GNU General Public License version 3 or            |
+ | any later version with exceptions for skins & plugins.                |
+ | See the README file for a full license statement.                     |
+ +-----------------------------------------------------------------------+
+ | Author: Thomas Bruederli <roundcube@gmail.com>                        |
+ +-----------------------------------------------------------------------+
+*/
+
+if (!class_exists('rcmail_install', false) || !isset($RCI)) {
     die("Not allowed! Please open installer/index.php instead.");
 }
 
-?>
-<form action="index.php" method="get">
-<?php
-
-$required_php_exts = array(
+$required_php_exts = [
     'PCRE'      => 'pcre',
     'DOM'       => 'dom',
     'Session'   => 'session',
     'XML'       => 'xml',
+    'Intl'      => 'intl',
     'JSON'      => 'json',
     'PDO'       => 'PDO',
     'Multibyte' => 'mbstring',
     'OpenSSL'   => 'openssl',
-);
+    'Filter'    => 'filter',
+    'Ctype'     => 'ctype',
+];
 
-$optional_php_exts = array(
+$optional_php_exts = [
+    'cURL'      => 'curl',
     'FileInfo'  => 'fileinfo',
-    'Libiconv'  => 'iconv',
-    'Intl'      => 'intl',
     'Exif'      => 'exif',
+    'Iconv'     => 'iconv',
     'LDAP'      => 'ldap',
     'GD'        => 'gd',
     'Imagick'   => 'imagick',
-);
+    'Zip'       => 'zip',
+];
 
-$required_libs = array(
+$required_libs = [
     'PEAR'      => 'pear.php.net',
     'Auth_SASL' => 'pear.php.net',
     'Net_SMTP'  => 'pear.php.net',
-    'Net_IDNA2' => 'pear.php.net',
     'Mail_mime' => 'pear.php.net',
-);
+    'GuzzleHttp\Client' => 'github.com/guzzle/guzzle',
+];
 
-$optional_libs = array(
+$optional_libs = [
     'Net_LDAP3' => 'git.kolab.org',
-);
+];
 
-$ini_checks = array(
-    'file_uploads'                  => 1,
-    'session.auto_start'            => 0,
-    'mbstring.func_overload'        => 0,
-    'suhosin.session.encrypt'       => 0,
-);
+$ini_checks = [
+    'file_uploads'            => 1,
+    'session.auto_start'      => 0,
+    'mbstring.func_overload'  => 0,
+    'suhosin.session.encrypt' => 0,
+];
 
-$optional_checks = array(
-    // required for utils/modcss.inc, should we require this?
-    'allow_url_fopen'  => 1,
-    'date.timezone'    => '-VALID-',
-);
+$optional_checks = [
+    'date.timezone' => '-VALID-',
+];
 
-$source_urls = array(
-    'Sockets'   => 'http://www.php.net/manual/en/book.sockets.php',
-    'Session'   => 'http://www.php.net/manual/en/book.session.php',
-    'PCRE'      => 'http://www.php.net/manual/en/book.pcre.php',
-    'FileInfo'  => 'http://www.php.net/manual/en/book.fileinfo.php',
-    'Libiconv'  => 'http://www.php.net/manual/en/book.iconv.php',
-    'Multibyte' => 'http://www.php.net/manual/en/book.mbstring.php',
-    'OpenSSL'   => 'http://www.php.net/manual/en/book.openssl.php',
-    'JSON'      => 'http://www.php.net/manual/en/book.json.php',
-    'DOM'       => 'http://www.php.net/manual/en/book.dom.php',
-    'Intl'      => 'http://www.php.net/manual/en/book.intl.php',
-    'Exif'      => 'http://www.php.net/manual/en/book.exif.php',
-    'oci8'      => 'http://www.php.net/manual/en/book.oci8.php',
-    'PDO'       => 'http://www.php.net/manual/en/book.pdo.php',
-    'LDAP'      => 'http://www.php.net/manual/en/book.ldap.php',
-    'GD'        => 'http://www.php.net/manual/en/book.image.php',
-    'Imagick'   => 'http://www.php.net/manual/en/book.imagick.php',
-    'pdo_mysql'   => 'http://www.php.net/manual/en/ref.pdo-mysql.php',
-    'pdo_pgsql'   => 'http://www.php.net/manual/en/ref.pdo-pgsql.php',
-    'pdo_sqlite'  => 'http://www.php.net/manual/en/ref.pdo-sqlite.php',
-    'pdo_sqlite2' => 'http://www.php.net/manual/en/ref.pdo-sqlite.php',
-    'pdo_sqlsrv'  => 'http://www.php.net/manual/en/ref.pdo-sqlsrv.php',
-    'pdo_dblib'   => 'http://www.php.net/manual/en/ref.pdo-dblib.php',
-    'PEAR'      => 'http://pear.php.net',
-    'Net_SMTP'  => 'http://pear.php.net/package/Net_SMTP',
-    'Mail_mime' => 'http://pear.php.net/package/Mail_mime',
-    'Net_IDNA2' => 'http://pear.php.net/package/Net_IDNA2',
+$source_urls = [
+    'cURL'      => 'https://www.php.net/manual/en/book.curl.php',
+    'Sockets'   => 'https://www.php.net/manual/en/book.sockets.php',
+    'Session'   => 'https://www.php.net/manual/en/book.session.php',
+    'PCRE'      => 'https://www.php.net/manual/en/book.pcre.php',
+    'FileInfo'  => 'https://www.php.net/manual/en/book.fileinfo.php',
+    'Multibyte' => 'https://www.php.net/manual/en/book.mbstring.php',
+    'OpenSSL'   => 'https://www.php.net/manual/en/book.openssl.php',
+    'JSON'      => 'https://www.php.net/manual/en/book.json.php',
+    'DOM'       => 'https://www.php.net/manual/en/book.dom.php',
+    'Iconv'     => 'https://www.php.net/manual/en/book.iconv.php',
+    'Intl'      => 'https://www.php.net/manual/en/book.intl.php',
+    'Exif'      => 'https://www.php.net/manual/en/book.exif.php',
+    'oci8'      => 'https://www.php.net/manual/en/book.oci8.php',
+    'PDO'       => 'https://www.php.net/manual/en/book.pdo.php',
+    'LDAP'      => 'https://www.php.net/manual/en/book.ldap.php',
+    'GD'        => 'https://www.php.net/manual/en/book.image.php',
+    'Imagick'   => 'https://www.php.net/manual/en/book.imagick.php',
+    'Zip'       => 'https://www.php.net/manual/en/book.zip.php',
+    'Filter'    => 'https://www.php.net/manual/en/book.filter.php',
+    'Ctype'     => 'https://www.php.net/manual/en/book.ctype.php',
+    'pdo_mysql'   => 'https://www.php.net/manual/en/ref.pdo-mysql.php',
+    'pdo_pgsql'   => 'https://www.php.net/manual/en/ref.pdo-pgsql.php',
+    'pdo_sqlite'  => 'https://www.php.net/manual/en/ref.pdo-sqlite.php',
+    'pdo_sqlite2' => 'https://www.php.net/manual/en/ref.pdo-sqlite.php',
+    'pdo_sqlsrv'  => 'https://www.php.net/manual/en/ref.pdo-sqlsrv.php',
+    'pdo_dblib'   => 'https://www.php.net/manual/en/ref.pdo-dblib.php',
+    'PEAR'      => 'https://pear.php.net',
+    'Net_SMTP'  => 'https://pear.php.net/package/Net_SMTP',
+    'Mail_mime' => 'https://pear.php.net/package/Mail_mime',
     'Net_LDAP3' => 'https://git.kolab.org/diffusion/PNL',
-);
+];
 
+?>
+<form action="index.php" method="get">
+
+<?php
 echo '<input type="hidden" name="_step" value="' . ($RCI->configured ? 3 : 2) . '" />';
 ?>
 
 <h3>Checking PHP version</h3>
 <?php
 
-define('MIN_PHP_VERSION', '5.4.0');
+define('MIN_PHP_VERSION', '5.5.0');
 if (version_compare(PHP_VERSION, MIN_PHP_VERSION, '>=')) {
     $RCI->pass('Version', 'PHP ' . PHP_VERSION . ' detected');
-} else {
+}
+else {
     $RCI->fail('Version', 'PHP Version ' . MIN_PHP_VERSION . ' or greater is required ' . PHP_VERSION . ' detected');
 }
 ?>
@@ -105,11 +125,12 @@ if (version_compare(PHP_VERSION, MIN_PHP_VERSION, '>=')) {
 // get extensions location
 $ext_dir = ini_get('extension_dir');
 
-$prefix = (PHP_SHLIB_SUFFIX === 'dll') ? 'php_' : '';
+$prefix = PHP_SHLIB_SUFFIX === 'dll' ? 'php_' : '';
 foreach ($required_php_exts as $name => $ext) {
     if (extension_loaded($ext)) {
         $RCI->pass($name);
-    } else {
+    }
+    else {
         $_ext = $ext_dir . '/' . $prefix . $ext . '.' . PHP_SHLIB_SUFFIX;
         $msg = @is_readable($_ext) ? 'Could be loaded. Please add in php.ini' : '';
         $RCI->fail($name, $msg, $source_urls[$name]);
@@ -136,13 +157,12 @@ foreach ($optional_php_exts as $name => $ext) {
 
 ?>
 
-
 <h3>Checking available databases</h3>
 <p class="hint">Check which of the supported extensions are installed. At least one of them is required.</p>
 
 <?php
 
-$prefix = (PHP_SHLIB_SUFFIX === 'dll') ? 'php_' : '';
+$prefix = PHP_SHLIB_SUFFIX === 'dll' ? 'php_' : '';
 foreach ($RCI->supported_dbs as $database => $ext) {
     if (extension_loaded($ext)) {
         $RCI->pass($database);
@@ -161,7 +181,6 @@ if (empty($found_db_driver)) {
 
 ?>
 
-
 <h3>Check for required 3rd party libs</h3>
 <p class="hint">This also checks if the include path is set correctly.</p>
 
@@ -223,7 +242,8 @@ foreach ($optional_checks as $var => $val) {
     if ($val === '-NOTEMPTY-') {
         if (empty($status)) {
             $RCI->optfail($var, "Could be set");
-        } else {
+        }
+        else {
             $RCI->pass($var);
         }
         echo '<br />';
@@ -256,7 +276,7 @@ foreach ($optional_checks as $var => $val) {
 <?php
 
 if ($RCI->failures) {
-  echo '<p class="warning">Sorry but your webserver does not meet the requirements for Roundcube!<br />
+    echo '<p class="warning">Sorry but your webserver does not meet the requirements for Roundcube!<br />
             Please install the missing modules or fix the php.ini settings according to the above check results.<br />
             Hint: only checks showing <span class="fail">NOT OK</span> need to be fixed.</p>';
 }
