@@ -1,9 +1,10 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2005-2013, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -16,7 +17,6 @@
  +-----------------------------------------------------------------------+
 */
 
-
 /**
  * Class for HTML code creation
  *
@@ -26,14 +26,15 @@
 class html
 {
     protected $tagname;
-    protected $attrib = array();
-    protected $allowed = array();
     protected $content;
+    protected $attrib  = [];
+    protected $allowed = [];
 
     public static $doctype = 'xhtml';
     public static $lc_tags = true;
-    public static $common_attrib = array('id','class','style','title','align','unselectable','tabindex','role');
-    public static $containers = array('iframe','div','span','p','h1','h2','h3','ul','form','textarea','table','thead','tbody','tr','th','td','style','script');
+    public static $common_attrib = ['id','class','style','title','align','unselectable','tabindex','role'];
+    public static $containers    = ['iframe','div','span','p','h1','h2','h3','ul','form','textarea','table','thead','tbody','tr','th','td','style','script','a'];
+    public static $bool_attrib   = ['checked','multiple','disabled','selected','autofocus','readonly','required'];
 
 
     /**
@@ -41,7 +42,7 @@ class html
      *
      * @param array $attrib Hash array with tag attributes
      */
-    public function __construct($attrib = array())
+    public function __construct($attrib = [])
     {
         if (is_array($attrib)) {
             $this->attrib = $attrib;
@@ -65,42 +66,46 @@ class html
      *
      * @param string $tagname Tag name
      * @param array  $attrib  Tag attributes as key/value pairs
-     * @param string $content Optinal Tag content (creates a container tag)
-     * @param array  $allowed_attrib List with allowed attributes, omit to allow all
+     * @param string $content Optional Tag content (creates a container tag)
+     * @param array  $allowed List with allowed attributes, omit to allow all
+     *
      * @return string The XHTML tag
      */
-    public static function tag($tagname, $attrib = array(), $content = null, $allowed_attrib = null)
+    public static function tag($tagname, $attrib = [], $content = null, $allowed = null)
     {
-        if (is_string($attrib))
-            $attrib = array('class' => $attrib);
+        if (is_string($attrib)) {
+            $attrib = ['class' => $attrib];
+        }
 
-        $inline_tags = array('a','span','img');
-        $suffix = $attrib['nl'] || ($content && $attrib['nl'] !== false && !in_array($tagname, $inline_tags)) ? "\n" : '';
+        $inline_tags = ['a', 'span', 'img'];
+        $suffix = (isset($attrib['nl']) && $content && $attrib['nl'] && !in_array($tagname, $inline_tags)) ? "\n" : '';
 
         $tagname = self::$lc_tags ? strtolower($tagname) : $tagname;
         if (isset($content) || in_array($tagname, self::$containers)) {
-            $suffix = $attrib['noclose'] ? $suffix : '</' . $tagname . '>' . $suffix;
+            $suffix = !empty($attrib['noclose']) ? $suffix : '</' . $tagname . '>' . $suffix;
             unset($attrib['noclose'], $attrib['nl']);
-            return '<' . $tagname  . self::attrib_string($attrib, $allowed_attrib) . '>' . $content . $suffix;
+            return '<' . $tagname  . self::attrib_string($attrib, $allowed) . '>' . $content . $suffix;
         }
         else {
-            return '<' . $tagname  . self::attrib_string($attrib, $allowed_attrib) . '>' . $suffix;
+            return '<' . $tagname  . self::attrib_string($attrib, $allowed) . '>' . $suffix;
         }
     }
 
     /**
+     * Return DOCTYPE tag of specified type
      *
+     * @param string $type Document type (html5, xhtml, 'xhtml-trans, xhtml-strict)
      */
     public static function doctype($type)
     {
-        $doctypes = array(
+        $doctypes = [
             'html5'        => '<!DOCTYPE html>',
             'xhtml'        => '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">',
             'xhtml-trans'  => '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">',
             'xhtml-strict' => '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">',
-        );
+        ];
 
-        if ($doctypes[$type]) {
+        if (!empty($doctypes[$type])) {
             self::$doctype = preg_replace('/-\w+$/', '', $type);
             return $doctypes[$type];
         }
@@ -109,149 +114,176 @@ class html
     }
 
     /**
-     * Derrived method for <div> containers
+     * Derived method for <div> containers
      *
      * @param mixed  $attr Hash array with tag attributes or string with class name
      * @param string $cont Div content
+     *
      * @return string HTML code
      * @see html::tag()
      */
     public static function div($attr = null, $cont = null)
     {
         if (is_string($attr)) {
-            $attr = array('class' => $attr);
+            $attr = ['class' => $attr];
         }
-        return self::tag('div', $attr, $cont, array_merge(self::$common_attrib, array('onclick')));
+
+        return self::tag('div', $attr, $cont, array_merge(self::$common_attrib, ['onclick']));
     }
 
     /**
-     * Derrived method for <p> blocks
+     * Derived method for <p> blocks
      *
      * @param mixed  $attr Hash array with tag attributes or string with class name
      * @param string $cont Paragraph content
+     *
      * @return string HTML code
      * @see html::tag()
      */
     public static function p($attr = null, $cont = null)
     {
         if (is_string($attr)) {
-            $attr = array('class' => $attr);
+            $attr = ['class' => $attr];
         }
+
         return self::tag('p', $attr, $cont, self::$common_attrib);
     }
 
     /**
-     * Derrived method to create <img />
+     * Derived method to create <img />
+     *
+     * @param string|array $attr Hash array with tag attributes or string with image source (src)
      *
-     * @param mixed $attr Hash array with tag attributes or string with image source (src)
      * @return string HTML code
      * @see html::tag()
      */
     public static function img($attr = null)
     {
         if (is_string($attr)) {
-            $attr = array('src' => $attr);
+            $attr = ['src' => $attr];
         }
-        return self::tag('img', $attr + array('alt' => ''), null, array_merge(self::$common_attrib,
-            array('src','alt','width','height','border','usemap','onclick','onerror')));
+
+        $allowed = ['src','alt','width','height','border','usemap','onclick','onerror','onload'];
+
+        return self::tag('img', $attr + ['alt' => ''], null, array_merge(self::$common_attrib, $allowed));
     }
 
     /**
-     * Derrived method for link tags
+     * Derived method for link tags
+     *
+     * @param string|array $attr Hash array with tag attributes or string with link location (href)
+     * @param string       $cont Link content
      *
-     * @param mixed  $attr Hash array with tag attributes or string with link location (href)
-     * @param string $cont Link content
      * @return string HTML code
      * @see html::tag()
      */
     public static function a($attr, $cont)
     {
         if (is_string($attr)) {
-            $attr = array('href' => $attr);
+            $attr = ['href' => $attr];
         }
-        return self::tag('a', $attr, $cont, array_merge(self::$common_attrib,
-            array('href','target','name','rel','onclick','onmouseover','onmouseout','onmousedown','onmouseup')));
+
+        $allowed = ['href','target','name','rel','onclick','onmouseover','onmouseout','onmousedown','onmouseup'];
+
+        return self::tag('a', $attr, $cont, array_merge(self::$common_attrib, $allowed));
     }
 
     /**
-     * Derrived method for inline span tags
+     * Derived method for inline span tags
+     *
+     * @param string|array $attr Hash array with tag attributes or string with class name
+     * @param string       $cont Tag content
      *
-     * @param mixed  $attr Hash array with tag attributes or string with class name
-     * @param string $cont Tag content
      * @return string HTML code
      * @see html::tag()
      */
     public static function span($attr, $cont)
     {
         if (is_string($attr)) {
-            $attr = array('class' => $attr);
+            $attr = ['class' => $attr];
         }
+
         return self::tag('span', $attr, $cont, self::$common_attrib);
     }
 
     /**
-     * Derrived method for form element labels
+     * Derived method for form element labels
+     *
+     * @param string|array $attr Hash array with tag attributes or string with 'for' attrib
+     * @param string       $cont Tag content
      *
-     * @param mixed  $attr Hash array with tag attributes or string with 'for' attrib
-     * @param string $cont Tag content
      * @return string HTML code
      * @see html::tag()
      */
     public static function label($attr, $cont)
     {
         if (is_string($attr)) {
-            $attr = array('for' => $attr);
+            $attr = ['for' => $attr];
         }
-        return self::tag('label', $attr, $cont, array_merge(self::$common_attrib, array('for')));
+
+        return self::tag('label', $attr, $cont, array_merge(self::$common_attrib, ['for','onkeypress']));
     }
 
     /**
-     * Derrived method to create <iframe></iframe>
+     * Derived method to create <iframe></iframe>
+     *
+     * @param string|array $attr Hash array with tag attributes or string with frame source (src)
+     * @param string       $cont Tag content
      *
-     * @param mixed $attr Hash array with tag attributes or string with frame source (src)
      * @return string HTML code
      * @see html::tag()
      */
     public static function iframe($attr = null, $cont = null)
     {
         if (is_string($attr)) {
-            $attr = array('src' => $attr);
+            $attr = ['src' => $attr];
         }
-        return self::tag('iframe', $attr, $cont, array_merge(self::$common_attrib,
-            array('src','name','width','height','border','frameborder','onload','allowfullscreen')));
+
+        $allowed = ['src','name','width','height','border','frameborder','onload','allowfullscreen'];
+
+        return self::tag('iframe', $attr, $cont, array_merge(self::$common_attrib, $allowed));
     }
 
     /**
-     * Derrived method to create <script> tags
+     * Derived method to create <script> tags
+     *
+     * @param string|array $attr Hash array with tag attributes or string with script source (src)
+     * @param string       $cont Javascript code to be placed as tag content
      *
-     * @param mixed $attr Hash array with tag attributes or string with script source (src)
-     * @param string $cont Javascript code to be placed as tag content
      * @return string HTML code
      * @see html::tag()
      */
     public static function script($attr, $cont = null)
     {
         if (is_string($attr)) {
-            $attr = array('src' => $attr);
-        }
-        if ($cont) {
-            if (self::$doctype == 'xhtml')
-                $cont = "\n/* <![CDATA[ */\n" . $cont . "\n/* ]]> */\n";
-            else
-                $cont = "\n" . $cont . "\n";
+            $attr = ['src' => $attr];
         }
 
-        return self::tag('script', $attr + array('type' => 'text/javascript', 'nl' => true),
-            $cont, array_merge(self::$common_attrib, array('src','type','charset')));
+        if ($cont) {
+            if (self::$doctype == 'xhtml') {
+                $cont = "/* <![CDATA[ */\n{$cont}\n/* ]]> */";
+            }
+
+            $cont = "\n{$cont}\n";
+        }
+
+        if (self::$doctype == 'xhtml') {
+            $attr += ['type' => 'text/javascript'];
+        }
+
+        return self::tag('script', $attr + ['nl' => true], $cont,
+            array_merge(self::$common_attrib, ['src', 'type', 'charset']));
     }
 
     /**
-     * Derrived method for line breaks
+     * Derived method for line breaks
+     *
+     * @param array $attrib Associative array with tag attributes
      *
      * @return string HTML code
      * @see html::tag()
      */
-    public static function br($attrib = array())
+    public static function br($attrib = [])
     {
         return self::tag('br', $attrib);
     }
@@ -259,18 +291,19 @@ class html
     /**
      * Create string with attributes
      *
-     * @param array $attrib Associative arry with tag attributes
+     * @param array $attrib  Associative array with tag attributes
      * @param array $allowed List of allowed attributes
+     *
      * @return string Valid attribute string
      */
-    public static function attrib_string($attrib = array(), $allowed = null)
+    public static function attrib_string($attrib = [], $allowed = null)
     {
         if (empty($attrib)) {
             return '';
         }
 
-        $allowed_f  = array_flip((array)$allowed);
-        $attrib_arr = array();
+        $allowed_f  = array_flip((array) $allowed);
+        $attrib_arr = [];
 
         foreach ($attrib as $key => $value) {
             // skip size if not numeric
@@ -298,13 +331,18 @@ class html
             }
 
             // attributes with no value
-            if (in_array($key, array('checked', 'multiple', 'disabled', 'selected', 'autofocus'))) {
-                if ($value) {
-                    $attrib_arr[] = $key . '="' . $key . '"';
+            if (in_array($key, self::$bool_attrib)) {
+                if (!empty($value)) {
+                    $value = $key;
+                    if (self::$doctype == 'xhtml') {
+                        $value .= '="' . $value . '"';
+                    }
+
+                    $attrib_arr[] = $value;
                 }
             }
             else {
-                $attrib_arr[] = $key . '="' . self::quote($value) . '"';
+                $attrib_arr[] = $key . '="' . self::quote((string) $value) . '"';
             }
         }
 
@@ -314,20 +352,24 @@ class html
     /**
      * Convert a HTML attribute string attributes to an associative array (name => value)
      *
-     * @param string Input string
+     * @param string $str Input string
+     *
      * @return array Key-value pairs of parsed attributes
      */
     public static function parse_attrib_string($str)
     {
-        $attrib = array();
-        $regexp = '/\s*([-_a-z]+)=(["\'])??(?(2)([^\2]*)\2|(\S+?))/Ui';
+        $attrib = [];
+        $html   = '<html>'
+            . '<head><meta http-equiv="Content-Type" content="text/html; charset=' . RCUBE_CHARSET . '" /></head>'
+            . '<body><div ' . rtrim($str, '/ ') . ' /></body>'
+            . '</html>';
 
-        preg_match_all($regexp, stripslashes($str), $regs, PREG_SET_ORDER);
+        $document = new DOMDocument('1.0', RCUBE_CHARSET);
+        @$document->loadHTML($html);
 
-        // convert attributes to an associative array (name => value)
-        if ($regs) {
-            foreach ($regs as $attr) {
-                $attrib[strtolower($attr[1])] = html_entity_decode($attr[3] . $attr[4]);
+        if ($node = $document->getElementsByTagName('div')->item(0)) {
+            foreach ($node->attributes as $name => $attr) {
+                $attrib[strtolower($name)] = $attr->nodeValue;
             }
         }
 
@@ -343,16 +385,12 @@ class html
      */
     public static function quote($str)
     {
-        static $flags;
-
-        if (!$flags) {
-            $flags = ENT_COMPAT;
-            if (defined('ENT_SUBSTITUTE')) {
-                $flags |= ENT_SUBSTITUTE;
-            }
+        // PHP8 does not like e.g. an array as htmlspecialchars() argument
+        if (!is_string($str)) {
+            return (string) $str;
         }
 
-        return @htmlspecialchars($str, $flags, RCUBE_CHARSET);
+        return @htmlspecialchars($str, ENT_COMPAT | ENT_SUBSTITUTE, RCUBE_CHARSET);
     }
 }
 
@@ -366,26 +404,26 @@ class html
 class html_inputfield extends html
 {
     protected $tagname = 'input';
-    protected $type = 'text';
-    protected $allowed = array(
+    protected $type    = 'text';
+    protected $allowed = [
         'type','name','value','size','tabindex','autocapitalize','required',
         'autocomplete','checked','onchange','onclick','disabled','readonly',
         'spellcheck','results','maxlength','src','multiple','accept',
-        'placeholder','autofocus',
-    );
+        'placeholder','autofocus','pattern','oninput'
+    ];
 
     /**
      * Object constructor
      *
      * @param array $attrib Associative array with tag attributes
      */
-    public function __construct($attrib = array())
+    public function __construct($attrib = [])
     {
         if (is_array($attrib)) {
             $this->attrib = $attrib;
         }
 
-        if ($attrib['type']) {
+        if (!empty($attrib['type'])) {
             $this->type = $attrib['type'];
         }
     }
@@ -393,8 +431,9 @@ class html_inputfield extends html
     /**
      * Compose input tag
      *
-     * @param string $value Field value
+     * @param string $value  Field value
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
     public function show($value = null, $attrib = null)
@@ -410,6 +449,7 @@ class html_inputfield extends html
         }
         // set type
         $this->attrib['type'] = $this->type;
+
         return parent::show();
     }
 }
@@ -434,9 +474,9 @@ class html_passwordfield extends html_inputfield
 class html_hiddenfield extends html
 {
     protected $tagname = 'input';
-    protected $type = 'hidden';
-    protected $fields_arr = array();
-    protected $allowed = array('type','name','value','onchange','disabled','readonly');
+    protected $type    = 'hidden';
+    protected $allowed = ['type','name','value','onchange','disabled','readonly'];
+    protected $fields  = [];
 
     /**
      * Constructor
@@ -457,7 +497,7 @@ class html_hiddenfield extends html
      */
     public function add($attrib)
     {
-        $this->fields_arr[] = $attrib;
+        $this->fields[] = $attrib;
     }
 
     /**
@@ -468,9 +508,11 @@ class html_hiddenfield extends html
     public function show()
     {
         $out = '';
-        foreach ($this->fields_arr as $attrib) {
-            $out .= self::tag($this->tagname, array('type' => $this->type) + $attrib);
+
+        foreach ($this->fields as $attrib) {
+            $out .= self::tag($this->tagname, ['type' => $this->type] + $attrib);
         }
+
         return $out;
     }
 }
@@ -490,6 +532,7 @@ class html_radiobutton extends html_inputfield
      *
      * @param string $value  Value of the checked field
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
     public function show($value = '', $attrib = null)
@@ -521,6 +564,7 @@ class html_checkbox extends html_inputfield
      *
      * @param string $value  Value of the checked field
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
     public function show($value = '', $attrib = null)
@@ -537,6 +581,38 @@ class html_checkbox extends html_inputfield
     }
 }
 
+/**
+ * Class to create HTML button
+ *
+ * @package    Framework
+ * @subpackage View
+ */
+class html_button extends html_inputfield
+{
+    protected $tagname = 'button';
+    protected $type    = 'button';
+
+    /**
+     * Get HTML code for this object
+     *
+     * @param string $content Text Content of the button
+     * @param array  $attrib  Additional attributes to override
+     *
+     * @return string HTML output
+     */
+    public function show($content = '', $attrib = null)
+    {
+        // overwrite object attributes
+        if (is_array($attrib)) {
+            $this->attrib = array_merge($this->attrib, $attrib);
+        }
+
+        $this->content = $content;
+
+        return parent::show();
+    }
+}
+
 /**
  * Class to create an HTML textarea
  *
@@ -546,14 +622,15 @@ class html_checkbox extends html_inputfield
 class html_textarea extends html
 {
     protected $tagname = 'textarea';
-    protected $allowed = array('name','rows','cols','wrap','tabindex',
-        'onchange','disabled','readonly','spellcheck');
+    protected $allowed = ['name','rows','cols','wrap','tabindex',
+        'onchange','disabled','readonly','spellcheck'];
 
     /**
      * Get HTML code for this object
      *
      * @param string $value  Textbox value
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
     public function show($value = '', $attrib = null)
@@ -586,16 +663,16 @@ class html_textarea extends html
  * Builder for HTML drop-down menus
  * Syntax:<pre>
  * // create instance. arguments are used to set attributes of select-tag
- * $select = new html_select(array('name' => 'fieldname'));
+ * $select = new html_select(['name' => 'fieldname']);
  *
  * // add one option
  * $select->add('Switzerland', 'CH');
  *
  * // add multiple options
- * $select->add(array('Switzerland','Germany'), array('CH','DE'));
+ * $select->add(['Switzerland','Germany'], ['CH','DE']);
  *
  * // generate pulldown with selection 'Switzerland'  and return html-code
- * // as second argument the same attributes available to instanciate can be used
+ * // as second argument the same attributes available to instantiate can be used
  * print $select->show('CH');
  * </pre>
  *
@@ -605,9 +682,9 @@ class html_textarea extends html
 class html_select extends html
 {
     protected $tagname = 'select';
-    protected $options = array();
-    protected $allowed = array('name','size','tabindex','autocomplete',
-        'multiple','onchange','disabled','rel');
+    protected $options = [];
+    protected $allowed = ['name','size','tabindex','autocomplete',
+        'multiple','onchange','disabled','rel'];
 
     /**
      * Add a new option to this drop-down
@@ -616,15 +693,18 @@ class html_select extends html
      * @param mixed $values Option value or array with option values
      * @param array $attrib Additional attributes for the option entry
      */
-    public function add($names, $values = null, $attrib = array())
+    public function add($names, $values = null, $attrib = [])
     {
         if (is_array($names)) {
             foreach ($names as $i => $text) {
-                $this->options[] = array('text' => $text, 'value' => $values[$i]) + $attrib;
+                $this->options[] = [
+                    'text'  => $text,
+                    'value' => isset($values[$i]) ? $values[$i] : $i
+                ] + $attrib;
             }
         }
         else {
-            $this->options[] = array('text' => $names, 'value' => $values) + $attrib;
+            $this->options[] = ['text' => $names, 'value' => $values] + $attrib;
         }
     }
 
@@ -633,9 +713,10 @@ class html_select extends html
      *
      * @param string $select Value of the selection option
      * @param array  $attrib Additional attributes to override
+     *
      * @return string HTML output
      */
-    public function show($select = array(), $attrib = null)
+    public function show($select = [], $attrib = null)
     {
         // overwrite object attributes
         if (is_array($attrib)) {
@@ -643,19 +724,22 @@ class html_select extends html
         }
 
         $this->content = "\n";
-        $select = (array)$select;
+        $select = (array) $select;
         foreach ($this->options as $option) {
-            $attr = array(
+            $attr = [
                 'value' => $option['value'],
                 'selected' => (in_array($option['value'], $select, true) ||
-                  in_array($option['text'], $select, true)) ? 1 : null);
+                    in_array($option['text'], $select, true)) ? 1 : null
+            ];
 
             $option_content = $option['text'];
             if (empty($this->attrib['is_escaped'])) {
                 $option_content = self::quote($option_content);
             }
 
-            $this->content .= self::tag('option', $attr + $option, $option_content, array('value','label','class','style','title','disabled','selected'));
+            $allowed = ['value','label','class','style','title','disabled','selected'];
+
+            $this->content .= self::tag('option', $attr + $option, $option_content, $allowed);
         }
 
         return parent::show();
@@ -672,11 +756,11 @@ class html_select extends html
 class html_table extends html
 {
     protected $tagname = 'table';
-    protected $allowed = array('id','class','style','width','summary',
-        'cellpadding','cellspacing','border');
+    protected $allowed = ['id','class','style','width','summary',
+        'cellpadding','cellspacing','border'];
 
-    private $header = array();
-    private $rows = array();
+    private $header   = null;
+    private $rows     = [];
     private $rowindex = 0;
     private $colindex = 0;
 
@@ -685,9 +769,9 @@ class html_table extends html
      *
      * @param array $attrib Named tag attributes
      */
-    public function __construct($attrib = array())
+    public function __construct($attrib = [])
     {
-        $default_attrib = self::$doctype == 'xhtml' ? array('summary' => '', 'border' => '0') : array();
+        $default_attrib = self::$doctype == 'xhtml' ? ['border' => '0'] : [];
         $this->attrib   = array_merge($attrib, $default_attrib);
 
         if (!empty($attrib['tagname']) && $attrib['tagname'] != 'table') {
@@ -705,17 +789,22 @@ class html_table extends html
     public function add($attr, $cont)
     {
         if (is_string($attr)) {
-            $attr = array('class' => $attr);
+            $attr = ['class' => $attr];
         }
 
         $cell = new stdClass;
         $cell->attrib  = $attr;
         $cell->content = $cont;
 
-        $this->rows[$this->rowindex]->cells[$this->colindex] = $cell;
-        $this->colindex += max(1, intval($attr['colspan']));
+        if (!isset($this->rows[$this->rowindex])) {
+            $this->rows[$this->rowindex] = new stdClass;
+            $this->rows[$this->rowindex]->attrib = [];
+        }
 
-        if ($this->attrib['cols'] && $this->colindex >= $this->attrib['cols']) {
+        $this->rows[$this->rowindex]->cells[$this->colindex] = $cell;
+        $this->colindex += max(1, isset($attr['colspan']) ? intval($attr['colspan']) : 0);
+
+        if (!empty($this->attrib['cols']) && $this->colindex >= $this->attrib['cols']) {
             $this->add_row();
         }
     }
@@ -723,41 +812,47 @@ class html_table extends html
     /**
      * Add a table header cell
      *
-     * @param array  $attr Cell attributes
-     * @param string $cont Cell content
+     * @param string|array $attr Cell attributes array or class name
+     * @param string       $cont Cell content
      */
     public function add_header($attr, $cont)
     {
         if (is_string($attr)) {
-            $attr = array('class' => $attr);
+            $attr = ['class' => $attr];
         }
 
         $cell = new stdClass;
-        $cell->attrib   = $attr;
-        $cell->content  = $cont;
-        $this->header[] = $cell;
+        $cell->attrib  = $attr;
+        $cell->content = $cont;
+
+        if (empty($this->header)) {
+            $this->header = new stdClass;
+            $this->header->attrib = [];
+        }
+
+        $this->header->cells[] = $cell;
     }
 
     /**
      * Remove a column from a table
      * Useful for plugins making alterations
      *
-     * @param string $class
+     * @param string $class Class name
      */
     public function remove_column($class)
     {
         // Remove the header
-        foreach ($this->header as $index=>$header){
-            if ($header->attrib['class'] == $class){
+        foreach ($this->header->cells as $index => $header) {
+            if ($header->attrib['class'] == $class) {
                 unset($this->header[$index]);
                 break;
             }
         }
 
         // Remove cells from rows
-        foreach ($this->rows as $i=>$row){
-            foreach ($row->cells as $j=>$cell){
-                if ($cell->attrib['class'] == $class){
+        foreach ($this->rows as $i => $row) {
+            foreach ($row->cells as $j => $cell) {
+                if ($cell->attrib['class'] == $class) {
                     unset($this->rows[$i]->cells[$j]);
                     break;
                 }
@@ -770,25 +865,43 @@ class html_table extends html
      *
      * @param array $attr Row attributes
      */
-    public function add_row($attr = array())
+    public function add_row($attr = [])
     {
         $this->rowindex++;
         $this->colindex = 0;
         $this->rows[$this->rowindex] = new stdClass;
         $this->rows[$this->rowindex]->attrib = $attr;
-        $this->rows[$this->rowindex]->cells = array();
+        $this->rows[$this->rowindex]->cells  = [];
+    }
+
+    /**
+     * Set header attributes
+     *
+     * @param string|array $attr Row attributes array or class name
+     */
+    public function set_header_attribs($attr = [])
+    {
+        if (is_string($attr)) {
+            $attr = ['class' => $attr];
+        }
+
+        if (empty($this->header)) {
+            $this->header = new stdClass;
+        }
+
+        $this->header->attrib = $attr;
     }
 
     /**
      * Set row attributes
      *
-     * @param array $attr  Row attributes
-     * @param int   $index Optional row index (default current row index)
+     * @param string|array $attr  Row attributes array or class name
+     * @param int          $index Optional row index (default current row index)
      */
-    public function set_row_attribs($attr = array(), $index = null)
+    public function set_row_attribs($attr = [], $index = null)
     {
         if (is_string($attr)) {
-            $attr = array('class' => $attr);
+            $attr = ['class' => $attr];
         }
 
         if ($index === null) {
@@ -796,7 +909,7 @@ class html_table extends html
         }
 
         // make sure row object exists (#1489094)
-        if (!$this->rows[$index]) {
+        if (empty($this->rows[$index])) {
             $this->rows[$index] = new stdClass;
         }
 
@@ -816,13 +929,14 @@ class html_table extends html
             $index = $this->rowindex;
         }
 
-        return $this->rows[$index] ? $this->rows[$index]->attrib : null;
+        return !empty($this->rows[$index]) ? $this->rows[$index]->attrib : null;
     }
 
     /**
      * Build HTML output of the table data
      *
      * @param array $attrib Table attributes
+     *
      * @return string The final table HTML code
      */
     public function show($attrib = null)
@@ -831,30 +945,39 @@ class html_table extends html
             $this->attrib = array_merge($this->attrib, $attrib);
         }
 
-        $thead = $tbody = "";
+        $thead        = '';
+        $tbody        = '';
+        $col_tagname  = $this->_col_tagname();
+        $row_tagname  = $this->_row_tagname();
+        $head_tagname = $this->_head_tagname();
 
         // include <thead>
         if (!empty($this->header)) {
             $rowcontent = '';
-            foreach ($this->header as $c => $col) {
-                $rowcontent .= self::tag($this->_head_tagname(), $col->attrib, $col->content);
+            foreach ($this->header->cells as $c => $col) {
+                $rowcontent .= self::tag($head_tagname, $col->attrib, $col->content);
             }
-            $thead = $this->tagname == 'table' ? self::tag('thead', null, self::tag('tr', null, $rowcontent, parent::$common_attrib)) :
-                self::tag($this->_row_tagname(), array('class' => 'thead'), $rowcontent, parent::$common_attrib);
+            $thead = $this->tagname == 'table' ? self::tag('thead', null, self::tag('tr', $this->header->attrib ?: null, $rowcontent, parent::$common_attrib)) :
+                self::tag($row_tagname, ['class' => 'thead'], $rowcontent, parent::$common_attrib);
         }
 
         foreach ($this->rows as $r => $row) {
             $rowcontent = '';
             foreach ($row->cells as $c => $col) {
-                $rowcontent .= self::tag($this->_col_tagname(), $col->attrib, $col->content);
+                if ($row_tagname == 'li' && empty($col->attrib) && count($row->cells) == 1) {
+                    $rowcontent .= $col->content;
+                }
+                else {
+                    $rowcontent .= self::tag($col_tagname, $col->attrib, $col->content);
+                }
             }
 
             if ($r < $this->rowindex || count($row->cells)) {
-                $tbody .= self::tag($this->_row_tagname(), $row->attrib, $rowcontent, parent::$common_attrib);
+                $tbody .= self::tag($row_tagname, $row->attrib, $rowcontent, parent::$common_attrib);
             }
         }
 
-        if ($this->attrib['rowsonly']) {
+        if (!empty($this->attrib['rowsonly'])) {
             return $tbody;
         }
 
@@ -862,13 +985,14 @@ class html_table extends html
         $this->content = $thead . ($this->tagname == 'table' ? self::tag('tbody', null, $tbody) : $tbody);
 
         unset($this->attrib['cols'], $this->attrib['rowsonly']);
+
         return parent::show();
     }
 
     /**
      * Count number of rows
      *
-     * @return The number of rows
+     * @return int The number of rows
      */
     public function size()
     {
@@ -880,7 +1004,7 @@ class html_table extends html
      */
     public function remove_body()
     {
-        $this->rows     = array();
+        $this->rows     = [];
         $this->rowindex = 0;
     }
 
@@ -889,8 +1013,8 @@ class html_table extends html
      */
     private function _row_tagname()
     {
-        static $row_tagnames = array('table' => 'tr', 'ul' => 'li', '*' => 'div');
-        return $row_tagnames[$this->tagname] ?: $row_tagnames['*'];
+        static $row_tagnames = ['table' => 'tr', 'ul' => 'li', '*' => 'div'];
+        return !empty($row_tagnames[$this->tagname]) ? $row_tagnames[$this->tagname] : $row_tagnames['*'];
     }
 
     /**
@@ -898,8 +1022,8 @@ class html_table extends html
      */
     private function _head_tagname()
     {
-        static $head_tagnames = array('table' => 'th', '*' => 'span');
-        return $head_tagnames[$this->tagname] ?: $head_tagnames['*'];
+        static $head_tagnames = ['table' => 'th', '*' => 'span'];
+        return !empty($head_tagnames[$this->tagname]) ? $head_tagnames[$this->tagname] : $head_tagnames['*'];
     }
 
     /**
@@ -907,8 +1031,7 @@ class html_table extends html
      */
     private function _col_tagname()
     {
-        static $col_tagnames = array('table' => 'td', '*' => 'span');
-        return $col_tagnames[$this->tagname] ?: $col_tagnames['*'];
+        static $col_tagnames = ['table' => 'td', '*' => 'span'];
+        return !empty($col_tagnames[$this->tagname]) ? $col_tagnames[$this->tagname] : $col_tagnames['*'];
     }
-
 }
