@@ -3,7 +3,8 @@
 /**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2012, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  | Copyright (c) 2005-2007, Jon Abernathy <jon@chuggnutt.com>            |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
@@ -135,9 +136,8 @@ class rcube_html2text
      * @var array $search
      * @see $replace
      */
-    protected $search = array(
+    protected $search = [
         '/\r/',                                  // Non-legal carriage return
-        '/^.*<body[^>]*>\n*/is',                 // Anything before <body>
         '/<head[^>]*>.*?<\/head>/is',            // <head>
         '/<script[^>]*>.*?<\/script>/is',        // <script>
         '/<style[^>]*>.*?<\/style>/is',          // <style>
@@ -157,7 +157,7 @@ class rcube_html2text
         '/(<table[^>]*>|<\/table>)/i',           // <table> and </table>
         '/(<tr[^>]*>|<\/tr>)/i',                 // <tr> and </tr>
         '/<td[^>]*>(.*?)<\/td>/i',               // <td> and </td>
-    );
+    ];
 
     /**
      * List of pattern replacements corresponding to patterns searched.
@@ -165,9 +165,8 @@ class rcube_html2text
      * @var array $replace
      * @see $search
      */
-    protected $replace = array(
+    protected $replace = [
         '',                                     // Non-legal carriage return
-        '',                                     // Anything before <body>
         '',                                     // <head>
         '',                                     // <script>
         '',                                     // <style>
@@ -187,7 +186,7 @@ class rcube_html2text
         "\n\n",                                 // <table> and </table>
         "\n",                                   // <tr> and </tr>
         "\t\t\\1\n",                            // <td> and </td>
-    );
+    ];
 
     /**
      * List of preg* regular expression patterns to search for,
@@ -196,10 +195,9 @@ class rcube_html2text
      * @var array $ent_search
      * @see $ent_replace
      */
-    protected $ent_search = array(
+    protected $ent_search = [
         '/&(nbsp|#160);/i',                      // Non-breaking space
-        '/&(quot|rdquo|ldquo|#8220|#8221|#147|#148);/i',
-                                         // Double quotes
+        '/&(quot|rdquo|ldquo|#8220|#8221|#147|#148);/i', // Double quotes
         '/&(apos|rsquo|lsquo|#8216|#8217);/i',   // Single quotes
         '/&gt;/i',                               // Greater-than
         '/&lt;/i',                               // Less-than
@@ -213,7 +211,7 @@ class rcube_html2text
         '/&(euro|#8364);/i',                     // Euro sign
         '/&(amp|#38);/i',                        // Ampersand: see _converter()
         '/[ ]{2,}/',                             // Runs of spaces, post-handling
-    );
+    ];
 
     /**
      * List of pattern replacements corresponding to patterns searched.
@@ -221,7 +219,7 @@ class rcube_html2text
      * @var array $ent_replace
      * @see $ent_search
      */
-    protected $ent_replace = array(
+    protected $ent_replace = [
         "\xC2\xA0",                             // Non-breaking space
         '"',                                    // Double quotes
         "'",                                    // Single quotes
@@ -237,7 +235,7 @@ class rcube_html2text
         'EUR',                                  // Euro sign. €
         '|+|amp|+|',                            // Ampersand: see _converter()
         ' ',                                    // Runs of spaces, post-handling
-    );
+    ];
 
     /**
      * List of preg* regular expression patterns to search for
@@ -245,26 +243,26 @@ class rcube_html2text
      *
      * @var array $callback_search
      */
-    protected $callback_search = array(
+    protected $callback_search = [
         '/<(a) [^>]*href=("|\')([^"\']+)\2[^>]*>(.*?)<\/a>/i', // <a href="">
         '/<(h)[123456]( [^>]*)?>(.*?)<\/h[123456]>/i',         // h1 - h6
         '/<(th)( [^>]*)?>(.*?)<\/th>/i',                       // <th> and </th>
-    );
+    ];
 
-   /**
-    * List of preg* regular expression patterns to search for in PRE body,
-    * used in conjunction with $pre_replace.
-    *
-    * @var array $pre_search
-    * @see $pre_replace
-    */
-    protected $pre_search = array(
+    /**
+     * List of preg* regular expression patterns to search for in PRE body,
+     * used in conjunction with $pre_replace.
+     *
+     * @var array $pre_search
+     * @see $pre_replace
+     */
+    protected $pre_search = [
         "/\n/",
         "/\t/",
         '/ /',
         '/<pre[^>]*>/',
         '/<\/pre>/'
-    );
+    ];
 
     /**
      * List of pattern replacements corresponding to patterns searched for PRE body.
@@ -272,13 +270,20 @@ class rcube_html2text
      * @var array $pre_replace
      * @see $pre_search
      */
-    protected $pre_replace = array(
+    protected $pre_replace = [
         '<br>',
         '&nbsp;&nbsp;&nbsp;&nbsp;',
         '&nbsp;',
         '',
         ''
-    );
+    ];
+
+    /**
+     * Temp. PRE content
+     *
+     * @var string $pre_content
+     */
+    protected $pre_content = '';
 
     /**
      * Contains a list of HTML tags to allow in the resulting text.
@@ -309,7 +314,7 @@ class rcube_html2text
      * @var array $_link_list
      * @see _build_link_list()
      */
-    protected $_link_list = array();
+    protected $_link_list = [];
 
     /**
      * Boolean flag, true if a table of link URLs should be listed after the text.
@@ -326,10 +331,10 @@ class rcube_html2text
      * will instantiate with that source propagated, all that has
      * to be done it to call get_text().
      *
-     * @param string  $source    HTML content
-     * @param boolean $from_file Indicates $source is a file to pull content from
-     * @param boolean $do_links  Indicate whether a table of link URLs is desired
-     * @param integer $width     Maximum width of the formatted text, 0 for no limit
+     * @param string $source    HTML content
+     * @param bool   $from_file Indicates $source is a file to pull content from
+     * @param bool   $do_links  Indicate whether a table of link URLs is desired
+     * @param int    $width     Maximum width of the formatted text, 0 for no limit
      */
     function __construct($source = '', $from_file = false, $do_links = true, $width = 75, $charset = 'UTF-8')
     {
@@ -347,8 +352,8 @@ class rcube_html2text
     /**
      * Loads source HTML into memory, either from $source string or a file.
      *
-     * @param string  $source    HTML content
-     * @param boolean $from_file Indicates $source is a file to pull content from
+     * @param string $source    HTML content
+     * @param bool   $from_file Indicates $source is a file to pull content from
      */
     function set_html($source, $from_file = false)
     {
@@ -425,7 +430,7 @@ class rcube_html2text
     protected function _convert()
     {
         // Variables used for building the link list
-        $this->_link_list = array();
+        $this->_link_list = [];
 
         $text = $this->html;
 
@@ -462,11 +467,19 @@ class rcube_html2text
         // Convert <PRE>
         $this->_convert_pre($text);
 
+        // Remove body tag and anything before
+        // We used to have '/^.*<body[^>]*>\n*/is' in $this->search, but this requires
+        // high pcre.backtrack_limit setting when converting long HTML strings (#8137)
+        if (($pos = stripos($text, '<body')) !== false) {
+            $pos = strpos($text, '>', $pos);
+            $text = substr($text, $pos + 1);
+        }
+
         // Run our defined tags search-and-replace
         $text = preg_replace($this->search, $this->replace, $text);
 
         // Run our defined tags search-and-replace with callback
-        $text = preg_replace_callback($this->callback_search, array($this, 'tags_preg_callback'), $text);
+        $text = preg_replace_callback($this->callback_search, [$this, 'tags_preg_callback'], $text);
 
         // Strip any other HTML tags
         $text = strip_tags($text, $this->allowed_tags);
@@ -491,13 +504,13 @@ class rcube_html2text
         $text = preg_replace("/\n\s+\n/", "\n\n", $text);
         $text = preg_replace("/[\n]{3,}/", "\n\n", $text);
 
-        // remove leading empty lines (can be produced by eg. P tag on the beginning)
+        // remove leading empty lines (can be produced by e.g. P tag on the beginning)
         $text = ltrim($text, "\n");
 
         // Wrap the text to a readable format
         // for PHP versions >= 4.0.2. Default width is 75
         // If width is 0 or less, don't wrap the text.
-        if ( $this->width > 0 ) {
+        if ($this->width > 0) {
             $text = wordwrap($text, $this->width);
         }
     }
@@ -515,7 +528,7 @@ class rcube_html2text
      */
     protected function _build_link_list($link, $display)
     {
-        if (!$this->_do_links || empty($link)) {
+        if (empty($link)) {
             return $display;
         }
 
@@ -540,6 +553,19 @@ class rcube_html2text
             $url .= "$link";
         }
 
+        if (!$this->_do_links) {
+            // When not using link list use URL if there's no content (#5795)
+            // The content here is HTML, convert it to text first
+            $h2t     = new rcube_html2text($display, false, false, 1024, $this->charset);
+            $display = $h2t->get_text();
+
+            if (empty($display) && preg_match('!^([a-z][a-z0-9.+-]+://)!i', $link)) {
+                return $link;
+            }
+
+            return $display;
+        }
+
         if (($index = array_search($url, $this->_link_list)) === false) {
             $index = count($this->_link_list);
             $this->_link_list[] = $url;
@@ -561,7 +587,7 @@ class rcube_html2text
 
             // Run our defined tags search-and-replace with callback
             $this->pre_content = preg_replace_callback($this->callback_search,
-                array($this, 'tags_preg_callback'), $this->pre_content);
+                [$this, 'tags_preg_callback'], $this->pre_content);
 
             // convert the content
             $this->pre_content = sprintf('<div><br>%s<br></div>',
@@ -569,7 +595,7 @@ class rcube_html2text
 
             // replace the content (use callback because content can contain $0 variable)
             $text = preg_replace_callback('/<pre[^>]*>.*<\/pre>/ismU',
-                array($this, 'pre_preg_callback'), $text, 1);
+                [$this, 'pre_preg_callback'], $text, 1);
 
             // free memory
             $this->pre_content = '';
@@ -583,10 +609,12 @@ class rcube_html2text
      */
     protected function _convert_blockquotes(&$text)
     {
-        $level = 0;
+        $level  = 0;
         $offset = 0;
+
         while (($start = stripos($text, '<blockquote', $offset)) !== false) {
             $offset = $start + 12;
+
             do {
                 $end = stripos($text, '</blockquote>', $offset);
                 $next = stripos($text, '<blockquote', $offset);
@@ -616,12 +644,12 @@ class rcube_html2text
                     // replace content with inner blockquotes
                     $this->_converter($body);
 
-                    // resore text width
+                    // restore text width
                     $this->width = $p_width;
 
                     // Add citation markers and create <pre> block
-                    $body = preg_replace_callback('/((?:^|\n)>*)([^\n]*)/', array($this, 'blockquote_citation_callback'), trim($body));
-                    $body = '<pre>' . htmlspecialchars($body) . '</pre>';
+                    $body = preg_replace_callback('/((?:^|\n)>*)([^\n]*)/', [$this, 'blockquote_citation_callback'], trim($body));
+                    $body = '<pre>' . htmlspecialchars($body, ENT_COMPAT | ENT_HTML401 | ENT_SUBSTITUTE, $this->charset) . '</pre>';
 
                     $text = substr_replace($text, $body . "\n", $start, $end + 13 - $start);
                     $offset = 0;
@@ -643,7 +671,7 @@ class rcube_html2text
     public function blockquote_citation_callback($m)
     {
         $line  = ltrim($m[2]);
-        $space = $line[0] == '>' ? '' : ' ';
+        $space = isset($line[0]) && $line[0] == '>' ? '' : ' ';
 
         return $m[1] . '>' . $space . $line;
     }
@@ -652,7 +680,8 @@ class rcube_html2text
      * Callback function for preg_replace_callback use.
      *
      * @param array $matches PREG matches
-     * @return string
+     *
+     * @return string Element content
      */
     public function tags_preg_callback($matches)
     {
@@ -672,7 +701,8 @@ class rcube_html2text
      * Callback function for preg_replace_callback use in PRE content handler.
      *
      * @param array $matches PREG matches
-     * @return string
+     *
+     * @return string PRE content
      */
     public function pre_preg_callback($matches)
     {
@@ -683,6 +713,7 @@ class rcube_html2text
      * Strtoupper function with HTML tags and entities handling.
      *
      * @param string $str Text to convert
+     *
      * @return string Converted text
      */
     private function _toupper($str)
@@ -704,6 +735,7 @@ class rcube_html2text
      * Strtoupper multibyte wrapper function with HTML entities handling.
      *
      * @param string $str Text to convert
+     *
      * @return string Converted text
      */
     private function _strtoupper($str)
