@@ -39,16 +39,23 @@ abstract class rcube_storage
      *
      * @var array
      */
-    public static $folder_types = array('drafts', 'sent', 'junk', 'trash');
+    public static $folder_types = ['drafts', 'sent', 'junk', 'trash'];
 
     protected $folder          = 'INBOX';
     protected $default_charset = 'ISO-8859-1';
-    protected $options         = array('auth_type' => 'check', 'language' => 'en_US');
+    protected $options         = ['auth_type' => 'check', 'language' => 'en_US'];
     protected $page_size       = 10;
     protected $list_page       = 1;
     protected $threading       = false;
     protected $search_set;
 
+    /**
+     * Internal (in-memory) cache
+     *
+     * @var array
+     */
+    protected $icache = [];
+
     /**
      * All (additional) headers used (in any way) by Roundcube
      * Not listed here: DATE, FROM, TO, CC, REPLY-TO, SUBJECT, CONTENT-TYPE, LIST-POST
@@ -56,18 +63,19 @@ abstract class rcube_storage
      *
      * @var array
      */
-    protected $all_headers = array(
-        'IN-REPLY-TO',
-        'BCC',
-        'SENDER',
-        'MESSAGE-ID',
+    protected $all_headers = [
         'CONTENT-TRANSFER-ENCODING',
-        'REFERENCES',
-        'X-DRAFT-INFO',
+        'BCC',
+        'IN-REPLY-TO',
         'MAIL-FOLLOWUP-TO',
         'MAIL-REPLY-TO',
+        'MESSAGE-ID',
+        'REFERENCES',
+        'RESENT-BCC',
         'RETURN-PATH',
-    );
+        'SENDER',
+        'X-DRAFT-INFO',
+    ];
 
     const UNKNOWN       = 0;
     const NOPERM        = 1;
@@ -91,7 +99,7 @@ abstract class rcube_storage
      * @param  integer  $port    Port to connect to
      * @param  string   $use_ssl SSL schema (either ssl or tls) or null if plain connection
      *
-     * @return boolean  TRUE on success, FALSE on failure
+     * @return bool True on success, False on failure
      */
     abstract function connect($host, $user, $pass, $port = 143, $use_ssl = null);
 
@@ -103,7 +111,7 @@ abstract class rcube_storage
     /**
      * Checks connection state.
      *
-     * @return boolean  TRUE on success, FALSE on failure
+     * @return bool True on success, False on failure
      */
     abstract function is_connected();
 
@@ -142,7 +150,7 @@ abstract class rcube_storage
      */
     public function set_options($opt)
     {
-        $this->options = array_merge($this->options, (array)$opt);
+        $this->options = array_merge($this->options, (array) $opt);
     }
 
     /**
@@ -160,7 +168,7 @@ abstract class rcube_storage
     /**
      * Activate/deactivate debug mode.
      *
-     * @param boolean $dbg True if conversation with the server should be logged
+     * @param bool $dbg True if conversation with the server should be logged
      */
     abstract function set_debug($dbg = true);
 
@@ -169,7 +177,7 @@ abstract class rcube_storage
      *
      * This will be used for message decoding if a charset specification is not available
      *
-     * @param  string $cs Charset string
+     * @param string $cs Charset string
      */
     public function set_charset($cs)
     {
@@ -180,7 +188,7 @@ abstract class rcube_storage
      * Set internal folder reference.
      * All operations will be performed on this folder.
      *
-     * @param  string $folder  Folder name
+     * @param string $folder Folder name
      */
     public function set_folder($folder)
     {
@@ -246,7 +254,7 @@ abstract class rcube_storage
     /**
      * Save a search result for future message listing methods.
      *
-     * @param  mixed  $set  Search set in driver specific format
+     * @param mixed $set Search set in driver specific format
      */
     abstract function set_search_set($set);
 
@@ -260,9 +268,9 @@ abstract class rcube_storage
     /**
      * Returns the storage server's (IMAP) capability
      *
-     * @param   string  $cap Capability name
+     * @param string $cap Capability name
      *
-     * @return  mixed   Capability value or TRUE if supported, FALSE if not
+     * @return mixed Capability value or True if supported, False if not
      */
     abstract function get_capability($cap);
 
@@ -270,19 +278,19 @@ abstract class rcube_storage
      * Sets threading flag to the best supported THREAD algorithm.
      * Enable/Disable threaded mode.
      *
-     * @param  boolean  $enable TRUE to enable and FALSE
+     * @param bool $enable True to enable threading
      *
-     * @return mixed   Threading algorithm or False if THREAD is not supported
+     * @return mixed Threading algorithm or False if THREAD is not supported
      */
     public function set_threading($enable = false)
     {
         $this->threading = false;
 
         if ($enable && ($caps = $this->get_capability('THREAD'))) {
-            $methods = array('REFS', 'REFERENCES', 'ORDEREDSUBJECT');
+            $methods = ['REFS', 'REFERENCES', 'ORDEREDSUBJECT'];
             $methods = array_intersect($methods, $caps);
 
-            $this->threading = array_shift($methods);
+            $this->threading = array_first($methods);
         }
 
         return $this->threading;
@@ -291,7 +299,7 @@ abstract class rcube_storage
     /**
      * Get current threading flag.
      *
-     * @return mixed  Threading algorithm or False if THREAD is not supported or disabled
+     * @return mixed Threading algorithm or False if THREAD is not supported or disabled
      */
     public function get_threading()
     {
@@ -302,9 +310,9 @@ abstract class rcube_storage
      * Checks the PERMANENTFLAGS capability of the current folder
      * and returns true if the given flag is supported by the server.
      *
-     * @param   string  $flag Permanentflag name
+     * @param string $flag Permanentflag name
      *
-     * @return  boolean True if this flag is supported
+     * @return bool True if this flag is supported
      */
     abstract function check_permflag($flag);
 
@@ -312,7 +320,7 @@ abstract class rcube_storage
      * Returns the delimiter that is used by the server
      * for folder hierarchy separation.
      *
-     * @return  string  Delimiter string
+     * @return string Delimiter string
      */
     abstract function get_hierarchy_delimiter();
 
@@ -321,20 +329,20 @@ abstract class rcube_storage
      *
      * @param string $name Namespace array index: personal, other, shared, prefix
      *
-     * @return  array  Namespace data
+     * @return array Namespace data
      */
     abstract function get_namespace($name = null);
 
     /**
      * Get messages count for a specific folder.
      *
-     * @param  string  $folder  Folder name
-     * @param  string  $mode    Mode for count [ALL|THREADS|UNSEEN|RECENT|EXISTS]
-     * @param  boolean $force   Force reading from server and update cache
-     * @param  boolean $status  Enables storing folder status info (max UID/count),
-     *                          required for folder_status()
+     * @param string $folder  Folder name
+     * @param string $mode    Mode for count [ALL|THREADS|UNSEEN|RECENT|EXISTS]
+     * @param bool   $force   Force reading from server and update cache
+     * @param bool   $status  Enables storing folder status info (max UID/count),
+     *                        required for folder_status()
      *
-     * @return int     Number of messages
+     * @return int Number of messages
      */
     abstract function count($folder = null, $mode = 'ALL', $force = false, $status = true);
 
@@ -381,7 +389,7 @@ abstract class rcube_storage
      * @param  string  $charset    Search charset
      * @param  string  $sort_field Header field to sort by
      *
-     * @todo: Search criteria should be provided in non-IMAP format, eg. array
+     * @todo: Search criteria should be provided in non-IMAP format, e.g. array
      */
     abstract function search($folder = null, $str = 'ALL', $charset = null, $sort_field = null);
 
@@ -437,7 +445,7 @@ abstract class rcube_storage
      * @param  rcube_message_part $o_part Part object created by get_structure()
      * @param  mixed              $print  True to print part, resource to write part contents in
      * @param  resource           $fp     File pointer to save the message part
-     * @param  boolean            $skip_charset_conv Disables charset conversion
+     * @param  bool               $skip_charset_conv Disables charset conversion
      *
      * @return string Message/part body if not printed
      */
@@ -446,7 +454,7 @@ abstract class rcube_storage
     /**
      * Fetch message body of a specific message from the server
      *
-     * @param  int    $uid  Message UID
+     * @param int $uid Message UID
      *
      * @return string $part Message/part body
      * @see    rcube_imap::get_message_part()
@@ -490,12 +498,12 @@ abstract class rcube_storage
     /**
      * Set message flag to one or several messages
      *
-     * @param mixed   $uids       Message UIDs as array or comma-separated string, or '*'
-     * @param string  $flag       Flag to set: SEEN, UNDELETED, DELETED, RECENT, ANSWERED, DRAFT, MDNSENT
-     * @param string  $folder     Folder name
-     * @param boolean $skip_cache True to skip message cache clean up
+     * @param mixed  $uids       Message UIDs as array or comma-separated string, or '*'
+     * @param string $flag       Flag to set: SEEN, UNDELETED, DELETED, RECENT, ANSWERED, DRAFT, MDNSENT
+     * @param string $folder     Folder name
+     * @param bool   $skip_cache True to skip message cache clean up
      *
-     * @return bool  Operation status
+     * @return bool Operation status
      */
     abstract function set_flag($uids, $flag, $folder = null, $skip_cache = false);
 
@@ -506,7 +514,7 @@ abstract class rcube_storage
      * @param string $flag    Flag to unset: SEEN, DELETED, RECENT, ANSWERED, DRAFT, MDNSENT
      * @param string $folder  Folder name
      *
-     * @return bool   Operation status
+     * @return bool Operation status
      * @see set_flag
      */
     public function unset_flag($uids, $flag, $folder = null)
@@ -521,13 +529,13 @@ abstract class rcube_storage
      * @param string|array $message The message source string or filename
      *                              or array (of strings and file pointers)
      * @param string       $headers Headers string if $message contains only the body
-     * @param boolean      $is_file True if $message is a filename
+     * @param bool         $is_file True if $message is a filename
      * @param array        $flags   Message flags
      * @param mixed        $date    Message internal date
      *
      * @return int|bool Appended message UID or True on success, False on error
      */
-    abstract function save_message($folder, &$message, $headers = '', $is_file = false, $flags = array(), $date = null);
+    abstract function save_message($folder, &$message, $headers = '', $is_file = false, $flags = [], $date = null);
 
     /**
      * Move message(s) from one folder to another.
@@ -536,7 +544,7 @@ abstract class rcube_storage
      * @param string $to    Target folder
      * @param string $from  Source folder
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function move_message($uids, $to, $from = null);
 
@@ -547,7 +555,7 @@ abstract class rcube_storage
      * @param string $to    Target folder
      * @param string $from  Source folder
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function copy_message($uids, $to, $from = null);
 
@@ -557,7 +565,7 @@ abstract class rcube_storage
      * @param mixed  $uids    Message UIDs as array or comma-separated string, or '*'
      * @param string $folder  Source folder
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function delete_message($uids, $folder = null);
 
@@ -566,9 +574,9 @@ abstract class rcube_storage
      *
      * @param mixed   $uids        Message UIDs as array or comma-separated string, or '*'
      * @param string  $folder      Folder name
-     * @param boolean $clear_cache False if cache should not be cleared
+     * @param bool    $clear_cache False if cache should not be cleared
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function expunge_message($uids, $folder = null, $clear_cache = true);
 
@@ -581,6 +589,8 @@ abstract class rcube_storage
      */
     protected function parse_uids($uids)
     {
+        $all = false;
+
         if ($uids === '*' || $uids === '1:*') {
             if (empty($this->search_set)) {
                 $uids = '1:*';
@@ -604,7 +614,7 @@ abstract class rcube_storage
             }
         }
 
-        return array($uids, (bool) $all);
+        return [$uids, $all];
     }
 
 
@@ -643,7 +653,7 @@ abstract class rcube_storage
      *
      * @param array $folders Folder name(s)
      *
-     * @return boolean True on success
+     * @return bool True on success
      */
     abstract function subscribe($folders);
 
@@ -652,7 +662,7 @@ abstract class rcube_storage
      *
      * @param array $folders Folder name(s)
      *
-     * @return boolean True on success
+     * @return bool True on success
      */
     abstract function unsubscribe($folders);
 
@@ -660,13 +670,13 @@ abstract class rcube_storage
      * Create a new folder on the server.
      *
      * @param string  $folder    New folder name
-     * @param boolean $subscribe True if the newvfolder should be subscribed
+     * @param bool    $subscribe True if the new folder should be subscribed
      * @param string  $type      Optional folder type (junk, trash, drafts, sent, archive)
-     * @param boolean $noselect  Make the folder \NoSelect folder by adding hierarchy
+     * @param bool    $noselect  Make the folder \NoSelect folder by adding hierarchy
      *                           separator at the end (useful for server that do not support
      *                           both folders and messages as folder children)
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function create_folder($folder, $subscribe = false, $type = null, $noselect = false);
 
@@ -676,7 +686,7 @@ abstract class rcube_storage
      * @param string $folder   Folder to rename
      * @param string $new_name New folder name
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function rename_folder($folder, $new_name);
 
@@ -685,7 +695,7 @@ abstract class rcube_storage
      *
      * @param string $folder Folder name
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function delete_folder($folder);
 
@@ -693,9 +703,9 @@ abstract class rcube_storage
      * Send expunge command and clear the cache.
      *
      * @param string  $folder      Folder name
-     * @param boolean $clear_cache False if cache should not be cleared
+     * @param bool    $clear_cache False if cache should not be cleared
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     public function expunge_folder($folder = null, $clear_cache = true)
     {
@@ -707,7 +717,7 @@ abstract class rcube_storage
      *
      * @param string  $folder  Folder name
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     public function clear_folder($folder = null)
     {
@@ -717,10 +727,10 @@ abstract class rcube_storage
     /**
      * Checks if folder exists and is subscribed
      *
-     * @param string   $folder       Folder name
-     * @param boolean  $subscription Enable subscription checking
+     * @param string  $folder       Folder name
+     * @param bool    $subscription Enable subscription checking
      *
-     * @return boolean True if folder exists, False otherwise
+     * @return bool True if folder exists, False otherwise
      */
     abstract function folder_exists($folder, $subscription = false);
 
@@ -779,7 +789,7 @@ abstract class rcube_storage
      *
      * @return int Folder status
      */
-    abstract function folder_status($folder = null, &$diff = array());
+    abstract function folder_status($folder = null, &$diff = []);
 
     /**
      * Synchronizes messages cache.
@@ -857,7 +867,7 @@ abstract class rcube_storage
         // getting config might be expensive, store special folders in memory
         if (!isset($this->icache['special-folders'])) {
             $rcube = rcube::get_instance();
-            $this->icache['special-folders'] = array();
+            $this->icache['special-folders'] = [];
 
             foreach (self::$folder_types as $type) {
                 if ($folder = $rcube->config->get($type . '_mbox')) {
@@ -899,7 +909,7 @@ abstract class rcube_storage
      * @param string $user    User name
      * @param string $acl     ACL string
      *
-     * @return boolean True on success, False on failure
+     * @return bool True on success, False on failure
      */
     abstract function set_acl($folder, $user, $acl);
 
@@ -911,7 +921,7 @@ abstract class rcube_storage
      * @param string $folder  Folder name
      * @param string $user    User name
      *
-     * @return boolean True on success, False on failure
+     * @return bool True on success, False on failure
      */
     abstract function delete_acl($folder, $user);
 
@@ -950,7 +960,7 @@ abstract class rcube_storage
      * @param string $folder  Folder name (empty for server metadata)
      * @param array  $entries Entry-value array (use NULL value as NIL)
      *
-     * @return boolean True on success, False on failure
+     * @return bool True on success, False on failure
      */
     abstract function set_metadata($folder, $entries);
 
@@ -960,7 +970,7 @@ abstract class rcube_storage
      * @param string $folder  Folder name (empty for server metadata)
      * @param array  $entries Entry names array
      *
-     * @return boolean True on success, False on failure
+     * @return bool True on success, False on failure
      */
     abstract function delete_metadata($folder, $entries);
 
@@ -974,7 +984,7 @@ abstract class rcube_storage
      *
      * @return array Metadata entry-value hash array on success, NULL on error
      */
-    abstract function get_metadata($folder, $entries, $options = array(), $force = false);
+    abstract function get_metadata($folder, $entries, $options = [], $force = false);
 
     /* -----------------------------------------
      *   Cache related functions
@@ -984,7 +994,7 @@ abstract class rcube_storage
      * Clears the cache.
      *
      * @param string  $key         Cache key name or pattern
-     * @param boolean $prefix_mode Enable it to clear all keys starting
+     * @param bool    $prefix_mode Enable it to clear all keys starting
      *                             with prefix specified in $key
      */
     abstract function clear_cache($key = null, $prefix_mode = false);
